@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: FilePicker.cpp 7967 2008-02-18 13:40:59Z chpe $
+ *  $Id: FilePicker.cpp 8155 2008-03-28 15:08:39Z chpe $
  */
 
 #include "mozilla-config.h"
@@ -357,8 +357,13 @@ NS_IMETHODIMP GFilePicker::GetFile(nsILocalFile **aFile)
 	return NS_OK;
 }
 
+#ifdef HAVE_GECKO_1_9
+/* readonly attribute nsIURI fileURL; */
+NS_IMETHODIMP GFilePicker::GetFileURL(nsIURI **aFileURL)
+#else
 /* readonly attribute nsIFileURL fileURL; */
 NS_IMETHODIMP GFilePicker::GetFileURL(nsIFileURL **aFileURL)
+#endif
 {
 	NS_ENSURE_TRUE (mDialog, NS_ERROR_FAILURE);
 
